import os
import json
import datetime

import requests
import singer


MAX_REQUEST_ITERATIONS = 50
BASE_FACEBOOK_GRAPH_API_URL = 'https://graph.facebook.com/v2.11/'
REACTIONS_URL = (
    '/posts?fields=created_time,story,message,'
    'reactions.limit(0).summary(1).as(total_reaction_count),'
    'reactions.type(NONE).limit(0).summary(1).as(none_reaction_count),'
    'reactions.type(LIKE).limit(0).summary(1).as(like_count),'
    'reactions.type(LOVE).limit(0).summary(1).as(love_count),'
    'reactions.type(HAHA).limit(0).summary(1).as(haha_count),'
    'reactions.type(WOW).limit(0).summary(1).as(wow_count),'
    'reactions.type(SAD).limit(0).summary(1).as(sad_count),'
    'reactions.type(ANGRY).limit(0).summary(1).as(angry_count),'
    'reactions.type(THANKFUL).limit(0).summary(1).as(thankful_count),'
    'reactions.type(PRIDE).limit(0).summary(1).as(pride_count),'
    'comments.limit(0).summary(1).as(comment_count)'
    '&limit=100&access_token='
)


def get_abs_path(path):
    return os.path.join(os.path.dirname(os.path.realpath(__file__)), path)


def load_schema(entity):
    return singer.utils.load_json(get_abs_path(
                                  "schemas/{}.json".format(entity)))


def format_datetime_string(original_dt):
    """Convert datetime into formatted string that is compatible w/ Singer spec.
    """
    FB_DATETIME_FORMAT = "%Y-%m-%dT%H:%M:%S+%f"
    TARGET_DATETIME_PARSE = "%Y-%m-%dT%H:%M:%SZ"
    dt = datetime.datetime.strptime(original_dt, FB_DATETIME_FORMAT)
    return dt.strftime(TARGET_DATETIME_PARSE)


def fetch_node_feed(node_id, access_token, *, after_state_marker=None):
    """Fetch the feed with all Post nodes for a given node.

    For more info, see
        https://developers.facebook.com/docs/graph-api/using-graph-api/

    node_id: str or int, unique identifier for a given node.
    access_token: str, access token to Facebook Graph API
    after_state_marker: If provided, will only fetch posts after that marker.
    """
    base_url = (BASE_FACEBOOK_GRAPH_API_URL + str(node_id) +
                REACTIONS_URL + access_token)
    if not after_state_marker:
        url = base_url
    else:
        url = base_url + '&after=' + after_state_marker
    response = requests.get(url)
    if response.status_code == 200:
        return json.loads(response.content.decode('utf-8'))
    else:
        error_message = (str(response.status_code) + ' Client Error: ' +
                         response.content.decode('utf-8'))
        raise requests.exceptions.HTTPError(error_message)


def clean_reactions_data(record):
    # Flatten the reactions data
    reactions = [
        'none_reaction_count', 'sad_count', 'like_count', 'love_count',
        'pride_count', 'total_reaction_count', 'haha_count', 'wow_count',
        'thankful_count', 'angry_count', 'comment_count',
    ]
    for key, value in record.items():
        if key in reactions:
            try:
                record[key] = record[key]['summary']['total_count']
            except TypeError:
                pass  # record is already clean
    return record


def write_records(data):
    for record in data:
        record['created_time'] = format_datetime_string(record['created_time'])
        record = clean_reactions_data(record)
        singer.write_record('facebook_posts', record)


def fetch_posts(node_id, access_token, after_state_marker=None):
    """Fetches all posts for a given node.

    node_id:  str or int, unique identifier for a given node.
    access_token: str, access token to Facebook Graph API
    after_state_marker: If provided, will only fetch posts after that marker.
    """
    current_state_marker = after_state_marker
    request_iterations_count = MAX_REQUEST_ITERATIONS
    schema = load_schema("facebook_posts")
    singer.write_schema("facebook_posts", schema, key_properties=["id"])

    node_feed = fetch_node_feed(node_id, access_token, after_state_marker=after_state_marker)
    write_records(node_feed['data'])
    try:
        while node_feed['paging']['cursors']['after'] and request_iterations_count:
            response_after_state_marker = node_feed['paging']['cursors']['after']
            # Did we already fetch the reponse marked by this state marker?
            if current_state_marker != response_after_state_marker:
                singer.write_state({'after': response_after_state_marker})
                node_feed = fetch_node_feed(
                    node_id, access_token,
                    after_state_marker=response_after_state_marker)
                write_records(node_feed['data'])
                request_iterations_count -= 1
            else:
                break
    except KeyError:
        pass



import argparse
import json

import singer

from .fetch_facebook_posts_data import fetch_posts


logger = singer.get_logger()


def create_parser():
    parser = argparse.ArgumentParser()

    parser.add_argument(
        '-c', '--config', help='Config file', required=True)
    parser.add_argument(
        '-s', '--state', help='State file')
    return parser


def main():
    parser = create_parser()
    args = parser.parse_args()

    with open(args.config) as config_file:
        config = json.load(config_file)
    required_config_vars = ['access_token', 'node_id', ]
    for r in required_config_vars:
        if r not in config:
            logger.fatal("Missing required configuration keys: {}".format(r))
            exit(1)

    after_state_marker = None
    if args.state:
        with open(args.state) as state_file:
            state = json.load(state_file)
            after_state_marker = state['after']

    try:
        fetch_posts(config['node_id'], access_token=config['access_token'],
                    after_state_marker=after_state_marker)
    except Exception as exception:
        logger.critical(exception)
        raise exception


if __name__ == "__main__":
    main()



from setuptools import setup, find_packages

setup(name="tap-facebook-posts",
      version="0.0.5",
      description="Singer.io tap for extracting posts data from the Facebook Graph API",
      author="rowanv",
      license='Apache 2.0',
      url="https://github.com/rowanv",
      classifiers=["Programming Language :: Python :: 3 :: Only"],
      py_modules=["tap_facebook_posts"],
      install_requires=[
        'python-dateutil==2.6.0',  # This is required by singer-python,
          # without this being here explicitly, there are dependency issues
        'requests',
        'singer-python==5.0.12',
      ],
      python_requires='>=3',
      entry_points="""
          [console_scripts]
          tap-facebook-posts=tap_facebook_posts:main
      """,
      packages=["tap_facebook_posts"],
      package_data = {
          "schemas": ["tap_facebook_posts/schemas/*.json"]
      },
      include_package_data=True,
)
